package com.flyfish.framework.repository.impl;

import com.flyfish.framework.domain.base.Qo;
import com.flyfish.framework.utils.CopyUtils;
import org.springframework.data.domain.Example;
import org.springframework.data.mongodb.core.query.Criteria;
import org.springframework.data.mongodb.core.query.Query;

import java.util.Optional;

abstract class QueryBuildUtils {

    /**
     * 从查询实体抽取内部查询信息
     *
     * @param qo 查询实体
     * @return 结果
     */
    static <T> Optional<Query> getQuery(Qo<T> qo) {
        Criteria criteria = null;
        if (null != qo.getCriteria()) {
            criteria = qo.getCriteria();
        } else if (null != qo.getExample()) {
            criteria = new Criteria().alike(qo.getExample());
        } else {
            Class<T> type = qo.pojoType();
            if (null != type && !Object.class.equals(type)) {
                try {
                    T pojo = CopyUtils.copyQueryProps(qo, qo.pojoType().newInstance());
                    criteria = new Criteria().alike(Example.of(pojo));
                } catch (InstantiationException | IllegalAccessException e) {
                    e.printStackTrace();
                }
            }
        }
        if (criteria != null) {
            // 针对删除状态全局筛选
            return Optional.of(new Query(Criteria.where("delete").ne(true).andOperator(criteria))
                    .with(qo.sorts()));
        }
        return Optional.empty();
    }
}
