package com.flyfish.framework.controller;

import com.flyfish.framework.bean.Result;
import com.flyfish.framework.domain.tree.TreeDomain;
import com.flyfish.framework.domain.tree.TreeQo;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.web.bind.annotation.GetMapping;

import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 支持树形解析的controller
 *
 * @param <T>
 * @param <Q>
 */
public abstract class TreeController<T extends TreeDomain<T>, Q extends TreeQo<T>> extends BaseController<T, Q> {

    /**
     * 获取权限树
     *
     * @return 结果
     */
    @GetMapping("tree")
    public Result<List<T>> getTree(Q qo) {
        qo.setRecursive(true);
        // 第一步，查询全部，并根据条件筛选
        List<T> filtered = service.getList(qo);
        // 第二步，根据父id组成map
        Map<String, List<T>> group = filtered.stream()
                .collect(Collectors.groupingBy(p -> StringUtils.defaultIfBlank(p.getParentId(), "")));
        // 第三步，筛选一级树深度
        List<T> topList = filtered.stream().filter(item -> null != item && TreeDomain.ROOT.equals(item.getParentId()))
                .collect(Collectors.toList());
        // 不是从根开始，直接查出
        if (CollectionUtils.isEmpty(topList)) {
            return Result.accept(filtered);
        }
        // 第三步，根据父id的map填充根tree
        return Result.accept(applyChildren(topList, group));
    }

    /**
     * 递归赋值儿子们
     *
     * @param children 儿子们
     * @param group    分组
     */
    private List<T> applyChildren(List<T> children, Map<String, List<T>> group) {
        if (CollectionUtils.isNotEmpty(children)) {
            children.forEach(child -> {
                List<T> treeNodes = group.getOrDefault(child.getId(), Collections.emptyList());
                child.setChildren(treeNodes);
                applyChildren(treeNodes, group);
            });
        }
        return children;
    }
}
